package com.rf.woodenFish.record.repostiory;

import com.rf.woodenFish.base.repository.BaseRepository;
import com.rf.woodenFish.record.model.UserKnockRecordEntity;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;

import java.util.List;
import java.util.Map;

/**
 * @Description:用户敲击记录管理
 * @Author: mimang
 * @Date: 2025/6/6
 */
public interface UserKnockRecordRepository extends BaseRepository<UserKnockRecordEntity,String> {

    /**
     * 获取今日最新敲击记录
     * @param openId
     * @param createDate
     * @return
     */
    UserKnockRecordEntity findTopByOpenIdAndCreateDateOrderByCreateTimeDesc(String openId,String createDate);

    /**
     * 当日功德
     * @param openId
     * @param createDate
     * @return
     */
    @Query(value = "SELECT SUM(knock_num) FROM t_user_knock_record WHERE open_id =?1 AND create_date = ?2 ",
            nativeQuery = true)
    Integer countByOpenIdAndCreateDate(String openId,String createDate);

    /**
     * 总功德
     * @param openId
     * @return
     */
    @Query(value = "SELECT SUM(knock_num) FROM t_user_knock_record WHERE open_id =?1  ",
            nativeQuery = true)
    Integer countByOpenId(String openId);

    /**
     * 当日功德排行
     * @param createDate
     * @return
     */
    @Query(value = "SELECT " +
            "ui.open_id," +
            "ui.user_name," +
            "ui.avatar," +
            "IFNULL(un.num,0) AS num " +
            "FROM t_user_info ui " +
            "LEFT JOIN " +
            "(" +
            "SELECT " +
            "ur.open_id, " +
            "SUM(ur.knock_num) AS num " +
            "FROM t_user_knock_record ur" +
            " WHERE " +
            "ur.create_date = ?1 " +
            "GROUP BY ur.open_id " +
            ") AS un " +
            "ON ui.open_id = un.open_id  " +
            "ORDER BY num DESC  ",
            nativeQuery = true)
    List<Map<String,Integer>> orderByCurrentDateNum(String createDate);

    /**
     * 总功德排行
     * @return
     */
    @Query(value = "SELECT " +
            "ui.open_id," +
            "ui.user_name," +
            "ui.avatar," +
            "IFNULL(un.num,0) AS num  " +
            "FROM t_user_info ui " +
            "LEFT JOIN " +
            "(" +
            "SELECT " +
            "ur.open_id, " +
            "SUM(ur.knock_num) AS num " +
            "FROM t_user_knock_record ur " +

            "GROUP BY ur.open_id " +
            ") AS un " +
            "ON ui.open_id = un.open_id  " +
            "ORDER BY num DESC  ",
            nativeQuery = true)
    List<Map<String,Integer>> orderByTotalNum();

    @Query(value = "SELECT " +
            " tr.create_date AS createDate, " +
            " SUM( tr.knock_num ) AS knockNum  " +
            "FROM " +
            " t_user_knock_record tr " +
            "WHERE " +
            " tr.open_id = ?1 " +
            "GROUP BY " +
            " tr.create_date  " +
            "ORDER BY " +
            " tr.create_date DESC",
            nativeQuery = true,
            countQuery = "SELECT " +
                    " COUNT( 1 )  " +
                    "FROM " +
                    " ( " +
                    " SELECT " +
                    "  tr.create_date AS createDate, " +
                    "  SUM( tr.knock_num ) AS knockNum  " +
                    " FROM " +
                    "  t_user_knock_record tr  " +
                    " WHERE " +
                    "  tr.open_id = ?1  " +
                    " GROUP BY " +
                    "  tr.create_date  " +
                    " ORDER BY " +
                    " tr.create_date DESC  " +
                    " ) AS a")
    Page<List<Map<String,String>>> findAllByOpenId(String openId, Pageable pageable);
}
