package com.rf.woodenFish.user.rest;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.rf.woodenFish.base.rest.BaseController;
import com.rf.woodenFish.user.dao.model.UserEntity;
import com.rf.woodenFish.user.service.UserService;
import com.rf.woodenFish.utils.Constant;
import com.rf.woodenFish.utils.DateUtil;
import com.rf.woodenFish.utils.JWTUtil;
import com.rf.woodenFish.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Random;

/**
 * @Description: 用户注册登录等相关接口
 * @Author: zsy
 * @Date: 2024/12/4
 */
@RestController
@RequestMapping("/user")
@Api(tags = "用户注册登录等相关接口")
@Slf4j
public class UserController extends BaseController {

    @Autowired
    private UserService userService;

    @GetMapping("/code2openid/{code}")
    @ApiOperation(value = "code 换 opernid",notes = "code：临时code，获取openid")
    public Result<JSONObject> code2openid(@PathVariable String code) throws Exception {
        String openId;
        HttpResponse response = getToken(code);
        int statusCode = response.getStatusLine().getStatusCode();//响应状态码
        String body = EntityUtils.toString(response.getEntity());
        Gson gson = new Gson();
        HashMap resultBody = gson.fromJson(body, HashMap.class);
        if(statusCode == 200 || statusCode == 204){
            openId = (String) resultBody.get("openid");
            if (openId == null){
                log.error("错误码：{}",resultBody.get("errcode"));
                log.error("错误消息：{}",resultBody.get("errmsg"));
                return fail(String.valueOf(resultBody.get("errmsg")));
            }
        }else{
            log.error("获取openid失败：错误码 "+resultBody.get("errcode")+"  "+resultBody.get("errmsg"));
            log.error("错误码：{}",resultBody.get("errcode"));
            log.error("错误消息：{}",resultBody.get("errmsg"));
            return fail();
        }
        JSONObject resultJson = new JSONObject();
        //静默注册登录
        UserEntity userEntity = this.userService.findByOpenId(openId);
        if(userEntity != null){
            resultJson.put("improve",false);
        }else{
            //注册
            userEntity = new UserEntity();
            userEntity.setCreateTime(DateUtil.getNowTime());
            userEntity.setUpdateTime(DateUtil.getNowTime());
            userEntity.setUserName("微信用户");
            userEntity.setOpenId(openId);
            userEntity.setPhone("-");
            userEntity.setGender("0");
            Random rand = new Random();
            int number = rand.nextInt(3) + 1;
            if (userEntity.getGender().equals("1")){
                userEntity.setAvatar("./userAvatar/man"+number+".png");
            }else {
                userEntity.setAvatar("./userAvatar/woman"+number+".png");
            }
            userEntity = this.userService.save(userEntity);
            resultJson.put("improve",true);
        }

        resultJson.put("token", JWTUtil.createToken(JSONObject.toJSONString(userEntity)));
        resultJson.put("user",userEntity);
        resultJson.put("openId",openId);
        log.info("响应消息："+resultJson.toJSONString());
        return success(resultJson);
    }

    public HttpResponse getToken(String code) throws IOException {
        String url = Constant.ASPI_URL +
                "?appid=" + Constant.ASPI_APPID +
                "&secret=" + Constant.ASPI_SECRET +
                "&js_code=" + code +
                "&grant_type=authorization_code";
        log.info("code2openid URL ："+url);
        HttpClient httpClient = HttpClients.createDefault();
        HttpGet httpGet = new HttpGet(url);
        httpGet.setHeader("Accept", "application/json");
        return httpClient.execute(httpGet);
    }

    /**
     * 修改用户信息
     * @param
     * @return
     */
    @ApiOperation(value = "修改用户信息接口",notes = "id:用户id,userName:昵称,phone:手机号,gender:性别(0-男；1-女),avatar:头像图片路径,intro:用户简介")
    @PostMapping("/updateUser")
    public Result updateAvatar(@RequestBody String json){
            UserEntity userEntity = JSONObject.parseObject(json, UserEntity.class);
            UserEntity user = this.userService.findById(userEntity.getId());
            if (user == null) {
                return fail(null, "用户不存在");
            }
            userEntity.setUpdateTime(DateUtil.getNowTime());
            userEntity.setOpenId(user.getOpenId());
            this.userService.save(userEntity);
            return success(userEntity);
    }

    /**
     * 上传图片
     * @param
     * @return
     */
    @ApiOperation(value = "上传图片",notes = "avatar:图片文件,userId:用户id,type:图片类型(1-用户头像；2-用户上传文件；3-转换后文件)")
    @PostMapping("/uploadPictures")
    public Result uploadPictures(String userId, String type, MultipartFile avatar){
        if (avatar == null) {
            return fail("头像为空");
        } else {
            UserEntity userEntity = userService.findById(userId);
            System.out.println("11:"+userEntity);
            if (userEntity == null) {
                return fail(null, "用户不存在");
            }
            String fileName = "";
            String FILEDIR = "";
            if (type.equals("1")) {
                FILEDIR = "./userAvatar/";
            }else if (type.equals("2")) {
                String date = DateUtil.getDateStr(new Date(), DateUtil.ACCURACY_PATTERN_MONTH);
                FILEDIR = "./userUpload/" + date + "/";
            }else {
                String date = DateUtil.getDateStr(new Date(), DateUtil.ACCURACY_PATTERN_MONTH);
                FILEDIR = "./userHandle/" + date + "/";
            }
            String avatarUrl = "";
            if (!avatar.isEmpty()) {
                fileName = userEntity.getOpenId() + "-=-" + System.currentTimeMillis() + "-=-" + avatar.getOriginalFilename();
                try {
                    File temp = new File(FILEDIR);
                    if (!temp.exists()) {
                        temp.mkdirs();
                    }
                    File fileLocal = new File(FILEDIR, fileName);
                    FileUtils.copyInputStreamToFile(avatar.getInputStream(), fileLocal);
                    avatarUrl = FILEDIR + fileName;
                    return success(avatarUrl);
                } catch (Exception e) {
                    e.printStackTrace();
                    return fail("文件上传失败");
                }
            } else {
                return fail("图片为空");
            }
        }
    }

    @GetMapping(value = "/show",produces = MediaType.IMAGE_JPEG_VALUE )
    @ResponseBody
    @ApiOperation(value = "图片展示",notes = "filePath：相对路径")
    public byte[] showPhoto(String filePath) throws Exception {
        try{
            File file = new File(filePath);
            if(file.exists()){
                FileInputStream inputStream = new FileInputStream(file);
                byte [] bytes = new byte[inputStream.available()];
                inputStream.read(bytes,0,inputStream.available());
                return bytes;
            }
            return null;
        }catch(Exception e){
            throw new Exception("未找到图片");
        }
    }

}
